<?php
// $Id: flexifilter.components.inc,v 1.2 2008/01/09 01:18:30 cwgordon7 Exp $

/**
 * Implementation of hook_flexifilter_components()
 *
 * @return An array of components to be used by the Flexifilter module. The keys of this array
 * are unique identifiers for the component (called the component class), and the values of the
 * array are again arrays, with the following keys:
 * - label : A human readable name for the component
 * - description (optional) : A human readable description of what the component does (defaults to none)
 * - is_container (optional) : TRUE if any of the #contains_ fields are TRUE (set automatically)
 * - contains_condition (optional) : TRUE if the component has a condition associated with it (defaults to FALSE)
 * - contains_components (optional) : TRUE if the component has children components (defaults to FALSE)
 * - callback : A callback function which implements the component
 * - callback_arguments (optional) : An array of arguments to pass to the callback function (defaults to none)
 * - group (optional) : A human readable name of the group that the component belongs to (defaults to "Other")
 */
function flexifilter_flexifilter_components() {
  $components = array();
  // If, While, etc.
  $components['flexifilter_control_if'] = array(
    'label' => t('If'),
    'description' => t('If and only if the condition is true, then the actions are performed.'),
    'contains_condition' => TRUE,
    'contains_components' => TRUE,
    'callback' => 'flexifilter_component_if',
    'group' => t('Control Structures'),
    'step' => 'both',
  );
  $components['flexifilter_control_while'] = array(
    'label' => t('While'),
    'description' => t('While the condition is true, the actions are performed.'),
    'contains_condition' => TRUE,
    'contains_components' => TRUE,
    'callback' => 'flexifilter_component_while',
    'group' => t('Control Structures'),
    'step' => 'both',
  );

  // Simple text operations
  $components['flexifilter_text_replace'] = array(
    'label' => t('Simple Text Replace'),
    'description' => t('Does a primtive search & replace on the text.'),
    'callback' => 'flexifilter_component_text_simple_replace',
    'group' => t('Text'),
    'step' => 'either',
  );
  $components['flexifilter_text_regex'] = array(
    'label' => t('Regex Text Replace'),
    'description' => t('Does a search & replace on the text using regular expressions.'),
    'callback' => 'flexifilter_component_text_regex_replace',
    'group' => t('Text'),
    'step' => 'either',
  );
  $components['flexifilter_text_alternation'] = array(
    'label' => t('Pattern-based Text Replacement'),
    'description' => t('Cycles through several replacements on the text.'),
    'callback' => 'flexifilter_component_text_pattern_replace',
    'group' => t('Text'),
    'step' => 'either',
  );
  $components['flexifilter_text_append'] = array(
    'label' => t('Simple Text Append'),
    'description' => t('Appends text to the end.'),
    'callback' => 'flexifilter_component_text_append',
    'group' => t('Text'),
    'step' => 'either',
  );
  $components['flexifilter_text_prepend'] = array(
    'label' => t('Simple Text Prepend'),
    'description' => t('Prepends text to the beginning.'),
    'callback' => 'flexifilter_component_text_prepend',
    'group' => t('Text'),
    'step' => 'either',
  );
  
  // Chunks!
  $components['flexifilter_chunk_grab'] = array(
    'label' => t('Chunk grabber.'),
    'description' => t('Grabs chunks of text for further filtering and then sticks the filtered version back into the text.'),
    'callback' => 'flexifilter_component_chunk_grab',
    'group' => t('Chunks'),
    'contains_components' => TRUE,
    'step' => 'either',
  );

  // Allow existing filters to be used as components
  foreach (module_list() as $module) {
    if ($module !== 'flexifilter') {
      $list = module_invoke($module, 'filter', 'list');
      if (isset($list) && is_array($list)) {
        foreach ($list as $delta => $name) {
          $components['flexifilter_existing__'. $module .'__filter__'. $delta] = array(
            'label' => $name,
            'description' => module_invoke($module, 'filter', 'description', $delta),
            'callback' => 'flexifilter_existing_filter_as_component',
            'callback_arguments' => array($module, $delta),
            'group' => t('Existing Filters'),
            'step' => 'both',
          );
        }
      }
    }
  }

  // Allow existing flexifilters to be used as components
  foreach (flexifilter_get_filters() as $fid => $filter) {
    $components['flexifilter_existing_flexifilter__'. $fid] = array(
      'label' => $filter['label'],
      'description' => $filter['description'],
      'callback' => 'flexifilter_existing_flexifilter_as_component',
      'callback_arguments' => array($fid),
      'group' => t('Existing Filters'),
      'step' => 'both',
    );
  }

  return $components;
}

function flexifilter_component_text_simple_replace($op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['find'] = array(
        '#type' => 'textfield',
        '#title' => t('Text to find'),
        '#default_value' => isset($settings['find']) ? $settings['find'] : '',
        '#required' => TRUE,
      );
      $form['replace'] = array(
        '#type' => 'textfield',
        '#title' => t('Replacement text'),
        '#default_value' => isset($settings['replace']) ? $settings['replace'] : '',
        '#required' => FALSE,
      );
      return $form;

    case 'prepare':
    case 'process':
      return str_replace($settings['find'], isset($settings['replace']) ? $settings['replace'] : '', $text);
  }
  return $text;
}

function flexifilter_component_text_regex_replace($op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['find'] = array(
        '#type' => 'textfield',
        '#title' => t('Text to find'),
        '#description' => t('Uses Perl compatible regular expressions'),
        '#default_value' => isset($settings['find']) ? $settings['find'] : '',
        '#required' => TRUE,
      );
      $form['replace'] = array(
        '#type' => 'textfield',
        '#title' => t('Replacement text'),
        '#description' => t('May contain references of the form $n'),
        '#default_value' => isset($settings['replace']) ? $settings['replace'] : '',
        '#required' => FALSE,
      );
      return $form;

    case 'prepare':
    case 'process':
      return preg_replace('~'. str_replace('~', '\~', $settings['find']) .'~', isset($settings['replace']) ? $settings['replace'] : '', $text);
  }
  return $text;
}

function flexifilter_component_if($op, $settings, $text) {
  switch ($op) {
    case 'settings':
      $form = array();
      return $form;

    case 'prepare':
    case 'process':
      if (flexifilter_invoke_condition($settings['condition'], $op, $text) == TRUE) {
        $text = flexifilter_invoke_components($settings['components'], $op, $text);
      }

    default:
      return $text;
  }
}

function flexifilter_component_while($op, $settings, $text) {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['limit'] = array(
        '#type' => 'textfield',
        '#title' => t('Number of iterations before abort'),
        '#description' => t('In order to prevent infinite looping, the loop will stop after a given number of iterations, even if the condition is still true.'),
        '#default_value' => isset($settings['limit']) ? $settings['limit'] : '100',
        '#required' => TRUE,
      );
      if (!isset($settings['limit'])) {
        $form['limit']['#value'] = $form['limit']['#default_value'];
      }
      return $form;

    case 'prepare':
    case 'process':
      /*
        Optimization case; the loop is skipped if none of the components do anything
        in this step. Also helps prevent naive infinite loops like this one:
        while (text contains "example") replace "example" with "e.g."
        As the replacement would only run in processing, the while loop would loop
        forever in preparation.
      */
      $limit = (integer)$settings['limit'];
      $n = 0;
      if (flexifilter_components_involve_step($settings['components'], $op)) {
        while (flexifilter_invoke_condition($settings['condition'], $op, $text) == TRUE) {
          $text = flexifilter_invoke_components($settings['components'], $op, $text);
          $n++;
          if ($n >= $limit) {
            drupal_set_message(t('Flexifilter while loop aborted at @count iterations. Consider tweaking the limit or condition.', array('@count' => $n)));
            break;
          }
        }
      }

    default:
      return $text;
  }
}

function _flexifilter_push_conf($settings) {
  global $conf;
  $conf_backup = array();
  foreach ($conf as $key => $value) {
    $conf_backup[$key] = $value;
  }
  foreach ($settings as $key => $value) {
    $conf[$key] = $value;
  }
  return $conf_backup;
}

function _flexifilter_pop_conf($old) {
  global $conf;
  $conf = $old;
}

function flexifilter_existing_filter_as_component($module, $delta, $op, $settings, $text = '') {
  global $conf;
  switch ($op) {
    case 'settings':
      $form = array();
      $oldconf = _flexifilter_push_conf($settings);
      $settings = module_invoke($module, 'filter', 'settings', $delta, 1, $text);
      _flexifilter_pop_conf($oldconf);
      if (is_array($settings) && count($settings) > 0) {
        foreach (element_children($settings) as $key) {
          $element = $settings[$key];
          if ($element['#type'] == 'fieldset') {
            foreach (element_children($element) as $key2) {
              $form[$key2] = $element[$key2];
            }
          }
        }
      }
      return $form;

    case 'prepare':
    case 'process':
      $oldconf = _flexifilter_push_conf($settings);
      $text = module_invoke($module, 'filter', $op, $delta, 1, $text);
      _flexifilter_pop_conf($oldconf);
      return $text;
  }
}

function flexifilter_existing_flexifilter_as_component($fid, $op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      return array();

    case 'prepare':
    case 'process':
      $filters = flexifilter_get_filters();
      $filter = $filters[$fid];
      return flexifilter_invoke_components($filter['components'], $op, $text);
  }
}

function flexifilter_component_text_pattern_replace($op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      $form = array();
      $max = isset($settings['number']) ? $settings['number'] : '2';
      $form['number'] = array(
        '#type' => 'textfield',
        '#title' => t('Number of replacements for text'),
        '#description' => t('Enter the number of replacement patterns for the given text.'),
        '#default_value' => $max,
        '#required' => TRUE,
      );
      $form['find'] = array(
        '#type' => 'textfield',
        '#title' => t('Text to find'),
        '#description' => t('Enter the text that will be replaced based on a pattern.'),
        '#default_value' => isset($settings['find']) ? $settings['find'] : '',
        '#required' => TRUE,
      );
      for ($i = 0; $i < $max; $i++) {
        $form['replace_'. $i] = array(
          '#type' => 'textfield',
          '#title' => t('Text to replace with (@num)', array('@num' => $i + 1)),
          '#description' => t('Enter the text that will replace the text specified in find on every run on replacement @round of every set of @max replacements.', array('@round' => $i + 1, '@max' => $max)),
          '#default_value' => isset($settings['replace_'. $i]) ? $settings['replace_'. $i] : '',
        );
      }
      return $form;

    case 'prepare':
    case 'process':
      $find = $settings['find'];
      $i = 0;
      $max = isset($settings['number']) ? $settings['number'] : '2';
      while (($pos = strpos($text, $find)) !== FALSE) {
        $text = flexifilter_component_text_pattern_replace_once($pos, $find, $settings['replace_'. $i], $text);
        $i++;
        if ($i >= $max) {
          // Reset $i.
          $i = $i - $max;
        }
      }
      return $text;
  }
}

/**
 * Helper function for flexifilter_component_text_pattern_replace. Replaces text only once.
 * Cannot use str_replace with $count because Drupal supports PHP 4.
 */
function flexifilter_component_text_pattern_replace_once($pos, $search, $replace, $subject) {
  return substr($subject, 0, $pos) . $replace . substr($subject, $pos + strlen($search));
}

function flexifilter_component_chunk_grab($op, $settings, $text) {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['starts'] = array(
        '#type' => 'textfield',
        '#title' => t('Chunk starts with'),
        '#description' => t('Enter the text that the chunk will start with.'),
        '#default_value' => isset($settings['starts']) ? $settings['starts'] : '[',
        '#required' => TRUE,
      );
      $form['ends'] = array(
        '#type' => 'textfield',
        '#title' => t('Chunk ends with'),
        '#description' => t('Enter the text that the chunk will end with.'),
        '#default_value' => isset($settings['ends']) ? $settings['ends'] : ']',
        '#required' => TRUE,
      );
      $form['pass_limits'] = array(
        '#type' => 'checkbox',
        '#title' => t('Pass delimiters'),
        '#description' => t('Should the delimiters be passed into the chunk?'),
        '#default_value' => isset($settings['pass_limits']) ? $settings['pass_limits'] : FALSE,
      );
      $form['case_sensitive'] = array(
        '#type' => 'checkbox',
        '#title' => t('Case sensitive'),
        '#description' => t('This option will make the chunk selection case-sensitive if checked.'),
        '#default_value' => isset($settings['case_sensitive']) ? $settings['case_sensitive'] : TRUE,
      );
      return $form;

    case 'prepare':
    case 'process':
      $start = $settings['starts'];
      $end = $settings['ends'];
      $pass_limits = $settings['pass_limits'];
      $case_sensitive = $settings['case_sensitive'];
      $chunk = '';
      $pos = 0;
      $endpos = 0;
      $tmp = '';
      $tmp2 = '';
      while ($pos !== FALSE && $endpos !== FALSE) {
        $pos = $case_sensitive ? strpos($text, $start) : stripos($text, $start);
        $tmp = substr($text, 0, $pos);
        if ($pos !== FALSE) {
          $endpos = $case_sensitive ? strpos($text, $end) : stripos($text, $end);
          if ($endpos !== FALSE) {
            $tmp2 = substr($text, $endpos + strlen($end));
            if ($pass_limits) {
              $endpos = $endpos + strlen($end);
            }
            else {
              $pos = $pos + strlen($start);
            }
            $chunk = substr($text, $pos, $endpos - $pos);
            $tmp .= flexifilter_invoke_components($settings['components'], $op, $chunk);
            $tmp .= $tmp2;
            $text = $tmp;
          }
        }
      }
      return $text;
  }
}

function flexifilter_component_text_append($op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['text'] = array(
	    '#type' => 'textfield',
        '#title' => t('Text to append'),
        '#description' => t('Enter the text to append.'),
        '#default_value' => isset($settings['text']) ? $settings['text'] : '',
      );
      return $form;

    case 'prepare':
    case 'process':
	  $append = isset($settings['text']) ? $settings['text'] : '';
      return $text . $append;
  }
}

function flexifilter_component_text_prepend($op, $settings, $text = '') {
  switch ($op) {
    case 'settings':
      $form = array();
      $form['text'] = array(
	    '#type' => 'textfield',
        '#title' => t('Text to prepend'),
        '#description' => t('Enter the text to prepend.'),
        '#default_value' => isset($settings['text']) ? $settings['text'] : '',
      );
      return $form;

    case 'prepare':
    case 'process':
	  $prepend = isset($settings['text']) ? $settings['text'] : '';
      return $settings['text'] . $prepend;
  }
}